#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "serial.h"


volatile unsigned char serial_flag, serial_start, serial_count, serial_buf[20];


ISR(USART1_RX_vect)
{
	char udr;

	udr = UDR1;
	serial_transmit(udr);

	if( !serial_start ) {
		if( !serial_flag ) {
			if( udr == 0x1B )	serial_flag = 1;
			else {
				serial_start = 1;
				serial_count = 0;

				serial_buf[serial_count++] = udr;
			}
		}
	}
	else if( serial_start ) {
		if( udr == 0x0D ) {
			serial_buf[serial_count] = 0x00;
			serial_flag = 1;
			serial_start = 0;
		}
		else {
			if( serial_count < 13 )		serial_buf[serial_count++] = udr;
		}
	}
}

void serial_transmit(unsigned char data)
{
	while( !( UCSR1A & (1 << UDRE1)) );

	UDR1 = data;
}

void serial_string(char *str)
{
	while( *str )	serial_transmit(*str++);
}

unsigned char serial_receive(void)
{
	while ( !(UCSR1A & (1 << RXC)) );

	return UDR1;
}

void serial_init(unsigned int baudrate)
{
	UCSR1A = 0x00;
	UCSR1B = 0x98;
	UCSR1C = 0x06;
	UBRR1H = baudrate >> 8;
	UBRR1L = baudrate & 0x0FF;

	serial_flag = 0;
	serial_start = 0;
}
